local TEXTURE = Resources.load_texture("Moonblade.png")
local AUDIO = Resources.load_audio("sfx.ogg")


function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_SWING")

	action:set_lockout(ActionLockout.new_animation())

	action.on_execute_func = function(self, user)
		local tile = user:current_tile()
		local slash = nil

		self:add_anim_action(3,
			function()
				local hilt = self:create_attachment("HILT")
				local hilt_sprite = hilt:sprite()
				hilt_sprite:set_texture(actor:texture())
				hilt_sprite:set_layer(-2)
				hilt_sprite:use_root_shader(true)

				local hilt_anim = hilt:animation()
				hilt_anim:copy_from(actor:animation())
				hilt_anim:set_state("HAND")
			end
		)

		if slash == nil then
			self:add_anim_action(3,
				function()
					slash = create_slash("DEFAULT", user, props)
					actor:field():spawn(slash, tile)
				end
			)
		end
	end
	return action
end

function spawn_slash_hitbox(user, user_tile, direction, field)
	user_tile:get_tile(direction, 1):set_highlight(Highlight.Solid)
	local hitbox_r = SharedHitbox.new(user, 60)
	hitbox_r:set_hit_props(user:copy_hit_props())
	field:spawn(hitbox_r, user_tile:get_tile(direction, 1))
end

function create_slash(animation_state, user, props)
	local spell = Spell.new(user:team())
	spell:set_texture(TEXTURE, true)
	spell:set_facing(user:facing())
	spell:set_hit_props(
		HitProps.new(
			props.damage,
			Hit.Impact | Hit.Flinch,
			Element.Sword,
			user:context(),
			Drag.None
		)
	)
	local anim = spell:animation()
	anim:load("Moonblade.animation")
	anim:set_state(animation_state)
	spell:animation():on_complete(
		function()
			spell:erase()
		end
	)
	local frame = 0
	local field = user:field()
	spell.on_update_func = function(self)
		local tile = self:current_tile()
		frame = frame + 1
		if (frame == 1) then
			spawn_slash_hitbox(spell, tile, Direction.Right, field)
		elseif (frame == 2) then
			spawn_slash_hitbox(spell, tile, Direction.DownRight, field)
		elseif (frame == 3) then
			spawn_slash_hitbox(spell, tile, Direction.Down, field)
		elseif (frame == 4) then
			spawn_slash_hitbox(spell, tile, Direction.DownLeft, field)
		elseif (frame == 5) then
			spawn_slash_hitbox(spell, tile, Direction.Left, field)
		elseif (frame == 6) then
			spawn_slash_hitbox(spell, tile, Direction.UpLeft, field)
		elseif (frame == 7) then
			spawn_slash_hitbox(spell, tile, Direction.Up, field)
		elseif (frame == 8) then
			spawn_slash_hitbox(spell, tile, Direction.UpRight, field)
		else
			spell:erase()
		end
	end
	spell.on_collision_func = function(self, other)
	end
	spell.on_attack_func = function(self, other)
	end

	spell.on_delete_func = function(self)
		self:erase()
	end

	spell.can_move_to_func = function(tile)
		return true
	end

	Resources.play_audio(AUDIO)

	return spell
end
